; $Id: ratios.pro,v 1.3 2008/05/07 20:32:19 bmy Exp $
;-----------------------------------------------------------------------
;+
; NAME:
;        RATIOS
;
; PURPOSE:
;        Creates ratio plots ( New/Old ) for GEOS-Chem tracers and OH. 
;
; CATEGORY:
;        GEOS-Chem Benchmarking
;
; CALLING SEQUENCE:
;        RATIOS, FILES, LEVELS, TAUS, VERSIONS, [, Keywords ]
;
; INPUTS:
;        FILES -> A 2-element vector containing the names of files
;             from the "old" and "new" GEOS-Chem model versions
;             that are to be compared. 
;
;        LEVELS -> A 4-element vector containing the level indices
;             for the GEOS-Chem surface layer and 500 hPa layer.
;             for both models (e.g. SFC_1, SFC_2, 500_1, 500_2).
;             NOTE: This is in Fortran notation (starting from 1!)
;
;        TAUS -> A 2-element vector contaning TAU values (hours GMT
;             from /1/1985) corresponding to the "old" and "new"
;             GEOS-Chem model versions.
;
;        TRACERS -> The list of transported tracers (i.e. diagnostic
;             category "IJ-AVG-$").
;
;        VERSIONS -> A 2-element vector containing the version
;             numbers for the "old" and "new" GEOS-Chem model
;             versions.
;
; KEYWORD PARAMETERS:
;        /DYNRANGE -> Set this switch to create plots using the whole
;             dynamic range of the data.  Default is to restrict
;             the plot range to +/- 5%.
;
;        /PS -> Set this switch to generate PostScript output.
;
;        OUTFILENAME -> If /PS is set, will write PostScript output 
;             to a file whose name is specified by this keyword.
;             Default is "tracer_ratio.pro".
;
; OUTPUTS:
;        None
;
; SUBROUTINES:
;        Internal Subroutines Included:
;        ===========================================
;        ComputeRatios   PlotRatio
;
;        External Subroutines Required:
;        ============================================
;        OPEN_DEVICE     CLOSE_DEVICE
;        MULTIPANEL      COLORBAR_NDIV    (function)
;        TVMAP           UNDEFINE 
;        CTM_GET_DATA,   EXTRACT_FILENAME (function)     
;     
; REQUIREMENTS:
;        References routines from both GAMAP and TOOLS packages.
;        
; NOTES:
;        (1) Meant to be called from BENCHMARK_1MON.
;
; EXAMPLES:
;        FILES    = [ 'ctm.bpch.v7-04-10', 'ctm.bpch.v7-04-11' ]
;        LEVELS   = [ 1, 1, 13, 13 ]
;        TAUS     = [ NYMD2TAU( 20010701 ), NYMD2TAU( 20010701 ) ]
;        TRACERS  = INDGEN( 43 ) + 1
;        VERSIONS = [ 'v7-04-10', 'v7-04-11' ]
; 
;        RATIOS, FILES, LEVELS, TAUS, TRACERS, VERSIONS, $
;             /PS, OUTFILENAME='myplot.ps'
;
;             ; Creates ratio plots of two GEOS-CHEM versions
;             ; (in this case v7-04-11 / v7-04-11) for July 2001.
;             ; Output is sent to PostScript file "myplot.ps".
;             ; The min & max of the data will be fixed at -/+ 5%.
;
;        RATIOS, FILES, LEVELS, TAUS, TRACERS, VERSIONS, $
;             /DYNRANGE, /PS, OUTFILENAME='myplot.ps'
;
;             ; Same as above, but this time the full dynamic range
;             ; of the data will be displayed.
;
;
; MODIFICATION HISTORY:
;        bmy, 14 Nov 2007: VERSION 1.01
;                          - based on older routine "tracer_ratio.pro"
;        bmy, 20 Nov 2007: VERSION 1.02
;                          - Now draw out-of-bounds triangles for
;                            the colorbar when using the "small"
;                            data ranges.  New feature of TVMAP.;
;        bmy, 07 May 2008: VERSION 1.06
;                          - Now allow for comparing models on 2
;                            different vertical grids.
;-
; Copyright (C) 2007-2008, 
; Bob Yantosca and Philippe Le Sager, Harvard University
; This software is provided as is without any warranty whatsoever. 
; It may be freely used, copied or distributed for non-commercial 
; purposes. This copyright notice must be kept with any copy of 
; this software. If this software shall be used commercially or 
; sold as part of a larger package, please contact the author.
; Bugs and comments should be directed to bmy@io.as.harvard.edu
; or phs@io.as.harvard.edu with subject "IDL routine ratios"
;-----------------------------------------------------------------------


pro ComputeRatios, Size, Data_Sfc_1, Data_Sfc_2, Ratio_Sfc, $
                         Data_500_1, Data_500_2, Ratio_500

   ;====================================================================
   ; Internal routine COMPUTERATIOS computes the ratio of "new" / "old"
   ; at both the surface level and the 500 hPa level. (bmy, 11/9/07)
   ;====================================================================

   ; Define ratio arrays
   Ratio_Sfc = FltArr( Size[0], Size[1] )
   Ratio_500 = FltArr( Size[0], Size[1] )

   ; Use brute force algorithm
   for J = 0L, Size[1]-1L do begin
   for I = 0L, Size[0]-1L do begin

      ;----------------------
      ; %%%%% SURFACE %%%%%
      ;----------------------
         
      ; Do the division "new" / "old"
      TmpRatio = Data_Sfc_2[I,J] / Data_Sfc_1[I,J]

      ; Reset to missing value if NaN or Infinity
      if ( not Finite( TmpRatio ) ) then TmpRatio = -9.99e30
         
      ; Store back into RATIO array
      Ratio_Sfc[I,J] = TmpRatio

      ;----------------------
      ; %%%%% 500 HPa %%%%%
      ;----------------------
         
      ; Do the division "new" / "old"
      TmpRatio = Data_500_2[I,J] / Data_500_1[I,J]

      ; Reset to missing value if NaN or Infinity
      if ( not Finite( TmpRatio ) ) then TmpRatio = -9.99e30
         
      ; Store back into RATIO array
      Ratio_500[I,J] = TmpRatio

   endfor
   endfor
end

;------------------------------------------------------------------------------

pro PlotRatio, Data, Level, TracerName, GridInfo, DynRange, _EXTRA=e
   
   ;====================================================================
   ; Internal routine PLOTRATIO plots either the surface or 500 hPa
   ; ratio of tracer between old and new versions (bmy, 11/9/07)
   ;====================================================================

   ; Plot title
   if ( Level gt 1 )                                    $
      then Title = 'Ratio @ 500 hPa for ' + TracerName $
      else Title = 'Ratio @ Surface for ' + TracerName  

   ; Number of colorbar tickmarks
   Divisions = ColorBar_NDiv( 6 )
      
   ; Don't plot the polar latitudes
   XMid      = GridInfo.XMid
   YMid      = GridInfo.YMid[ 1:GridInfo.JMX-2 ]
   Data      = Data[ *, 1:GridInfo.JMX-2 ]

   ; GOOD is the index of "good" data points
   Good      = Where( Data gt -9.99e30 )
   if ( Good[0] lt 0 ) then Message, 'No valid data points found!'

   ; If DYNRANGE is set, then plot the dynamic range of the data
   ; Otherwise, fix the min & max of the data as +/- 5% ratio
   if ( DynRange ) then begin
      MinData  =  Min( Data, Max=MaxData )
      ; COMMENT OUT -- center around zero 
      ;Extreme =  Max( [ Abs( MinData ), Abs( MaxData ) ] )
      ;MinData = -Extreme
      ;MaxData =  Extreme
   endif else begin
      MinData  = 0.95
      MaxData  = 1.05
      BotOut   = !MYCT.BOTTOM
      Triangle = 1
      NoGap    = 1
   endelse

   ; Plot data w/ country boundaries
   TvMap, Data, XMid, Ymid,                                   $
      /Countries,         /Coasts,         /Cbar,             $
      Division=Divisions, /Sample,         /Grid,             $
      Title=Title,        MinData=MinData, MaxData=MaxData,   $
      CBFormat='(f13.3)', BOR_Label=' ',   Triangle=Triangle, $
      NoGap=NoGap,        BotOut=BotOut,   _EXTRA=e

end

;------------------------------------------------------------------------------

pro Ratios, Files, Levels, Taus, Tracers, Versions, $
            DynRange=DynRange, PS=PS, OutFileName=OutFileName, _EXTRA=e
 
   ;====================================================================
   ; Initialization
   ;====================================================================
   
   ; External functions
   FORWARD_FUNCTION ColorBar_NDiv, Extract_FileName

   ; Arguments
   if ( N_Elements( Files       ) ne 2 ) then Message, 'Invalid FILES!'
   if ( N_Elements( Levels      ) ne 4 ) then Message, 'Invalid LEVELS'
   if ( N_Elements( Taus        ) ne 2 ) then Message, 'Invalid TAUS!'
   if ( N_Elements( Versions    ) ne 2 ) then Message, 'Invalid VERSIONS!'

   ; Keywords
   DynRange = Keyword_Set( DynRange )
   if ( N_Elements( OutFileName ) ne 1 ) then OutFileName = 'ratios.ps'

   ; Title for the top of the plot
   TopTitle = 'GEOS-Chem ' + Versions[1] +                                 $
      ' Tracer Ratios at Surface and 500 hPa!C!C' +                        $
      Extract_FileName( Files[1] ) +  ' / ' + Extract_FileName( Files[0] )

   ; Save original color table
   TvLct, R, G, B, /Get

   ; Load Blue-White-White-Red colortable
   MyCt, /BuWhWhRd
   
   ;====================================================================
   ; Read data from the files
   ;====================================================================
   
   ;------------------------------
   ; From FILE_1: the "old" file
   ;------------------------------

   ; Read tracers from the old file
   CTM_Get_Data, DataInfo_1, 'IJ-AVG-$', $
      File=Files[0], Tau0=Taus[0], Tracer=Tracers, /Quiet

   ; Read OH from the "new" file
   CTM_Get_Data, DataInfo, 'CHEM-L=$', $
      File=Files[0], Tau0=Taus[0], Tracer=1, /Quiet
   DataInfo_1 = [ DataInfo_1, DataInfo ]
   UnDefine, DataInfo

   ;------------------------------
   ; From FILE_2: the "new" file
   ;------------------------------

   ; Read tracers from the "new" file
   CTM_Get_Data, DataInfo_2, 'IJ-AVG-$', $
      File=Files[1], Tau0=Taus[1], Tracer=Tracers, /Quiet

   ; Read "OH" from the "new" file
   CTM_Get_Data, DataInfo, 'CHEM-L=$', $
      File=Files[1], Tau0=Taus[1], Tracer=1, /Quiet
   DataInfo_2 = [ DataInfo_2, DataInfo ]
   UnDefine, DataInfo

   ;------------------------------
   ; Error checks!
   ;------------------------------

   ; Stop if both DATAINFOs are incompatible
   if ( N_Elements( DataInfo_1 ) ne N_Elements( DataInfo_2 ) ) $
      then Message, 'Files have different numbers of tracers in them!'

   ;====================================================================
   ; Process data and create ratio plots!
   ;====================================================================

   ; Number of rows & colums on the plot
   Rows = 3
   Cols = 2

   ; Open the plot device and initialize the page
   Open_Device, /Color, Bits=8, /Portrait, PS=PS, File=OutFileName, _EXTRA=e

   ; Multiple panels per page
   MultiPanel, Rows=Rows, Cols=Cols, Margin=[ 0.03, 0.03, 0.03, 0.03 ]

   ; Loop over all data blocks
   for D = 0L,  N_Elements( DataInfo_1 )-1L do begin
   
      ;-----------------------------------------------------------------
      ; Error check grid, tracer name, and data block sizes 
      ;-----------------------------------------------------------------

      ; Get MODELINFO and GRIDINFO structures
      GetModelAndGridInfo, DataInfo_1[D], ModelInfo_1, GridInfo_1
      GetModelAndGridInfo, DataInfo_2[D], ModelInfo_2, GridInfo_2

      ; Make sure grids are compatible
      if ( GridInfo_1.IMX ne GridInfo_2.IMX OR  $
           GridInfo_1.JMX ne GridInfo_2.JMX   ) $
         then Message, 'Resolution mismatch!'

      ; Make sure the tracers correspond to each other
      TracerName_1 = DataInfo_1[D].TracerName
      TracerName_2 = DataInfo_2[D].TracerName
      if ( TracerName_1 ne TracerName_2 ) then Message, 'Tracer mismatch!'

      ; Get full-sized data arrays
      Data_1 = *( DataInfo_1[D].Data )
      Data_2 = *( DataInfo_2[D].Data )

      ; Get the dimensions of the data arrays
      Size_1 = Size( Data_1, /Dim )
      Size_2 = Size( Data_2, /Dim )

      ; Stop the run if the data block sizes don't agree
      if ( Size_1[0] ne Size_2[0] ) then Message, 'Longitude mismatch!'
      if ( Size_1[1] ne Size_2[1] ) then Message, 'Latitude mismatch!'

      ;-----------------------------------------------------------------
      ; Extract data arrays for surface and 500 hPa
      ;-----------------------------------------------------------------

      ; Surface data arrays
      Data_Sfc_1 = Data_1[ *, *, Levels[0]-1 ]
      Data_Sfc_2 = Data_2[ *, *, Levels[1]-1 ]

      ; 500 hPa data arrays
      Data_500_1 = Data_1[ *, *, Levels[2]-1 ]
      Data_500_2 = Data_2[ *, *, Levels[3]-1 ]

      ; We no longer need the 3-D data arrays
      UnDefine, Data_1
      UnDefine, Data_2

      ;-----------------------------------------------------------------
      ; Compute the ratios and plot the data
      ;-----------------------------------------------------------------

      ; Get the "new" / "old" ratios at surface and 500 hPa
      ComputeRatios, Size_1, Data_Sfc_1, Data_Sfc_2, Ratio_Sfc, $
                             Data_500_1, Data_500_2, Ratio_500

      ; Plot the Surface ratios
      PlotRatio, Ratio_Sfc,  Levels[0], TracerName_1, $
                 GridInfo_1, DynRange,  _EXTRA=e

      ; Plot the 500 hPa ratios
      PlotRatio, Ratio_500,  Levels[2], TracerName_1, $
                 GridInfo_1, DynRange,  _EXTRA=e
 
      ; Plot the top title on each page  
      if ( D*2 mod ( Rows * Cols ) eq 0 ) then begin
         XYoutS, 0.5, 1.03, TopTitle, $
            /Normal, Color=!MYCT.BLACK, CharSize=1.0, Align=0.5
      endif

      ;-----------------------------------------------------------------
      ; Undefine stuff for next iteration
      ;-----------------------------------------------------------------
      UnDefine, GridInfo_1
      UnDefine, GridInfo_2
      UnDefine, ModelInfo_1
      UnDefine, ModelInfo_2
      UnDefine, Size_1
      UnDefine, Size_2
      UnDefine, Ratio_Sfc
      UnDefine, Ratio_500
      UnDefine, TracerName_1
      UnDefine, TracerName_2

   endfor

   ;====================================================================
   ; Cleanup and quit
   ;====================================================================

   ; Close previous MULTIPANEL settings
   MultiPanel, /Off

   ; Close plot device
   Close_Device

   ; Restore original color table
   TvLct, R, G, B

   ; Quit
   return
end
 
